
local matrix = require("lua-matrix/lua/matrix")
local points = {}

local size = 100
local x = 150
local y = 200

local pX = matrix{
	{x},
	{x},
	{x+size},
	{x+size},
}
local pY = matrix{
	{y},
	{y+size},
	{y+size},
	{y},
}
local fX = matrix{
	{0},
	{0},
	{0},
	{0},
}
local fY = matrix{
	{0},
	{0},
	{0},
	{0},
}
local vX = matrix{
	{0},
	{0},
	{0},
	{0},
}
local vY = matrix{
	{0},
	{0},
	{0},
	{0},
}
local aX = matrix{
	{0},
	{0},
	{0},
	{0},
}
local aY = matrix{
	{0},
	{0},
	{0},
	{0},
}

-- Steifigkeit:
local c = 10
-- Dämpfung:
local d = 1

local m = 10

local M = matrix {
	{ m, 0, 0, 0 },
	{ 0, m, 0, 0 },
	{ 0, 0, m, 0 },
	{ 0, 0, 0, m } }
local D = matrix {
	{ 3*d, -d, -d, -d },
	{ -d, 3*d, -d, -d },
	{ -d, -d, 3*d, -d },
	{ -d, -d, -d, 3*d } }
local S = matrix {
	{ 3*c, -c, -c, -c },
	{ -c, 3*c, -c, -c },
	{ -c, -c, 3*c, -c },
	{ -c, -c, -c, 3*c } }

local diag = math.sqrt( size^2 + size^2 )
local l0X = matrix{
	{ size + size },
	{ size + size },
	{ -1*(size + size) },
	{ -1*(size + size) },
}
local l0Y = matrix{
	{ size + size },
	{ -1*(size + size) },
	{ -1*(size + size) },
	{ size + size },
}

function insertPoint( x, y )
end

function love.load()
	local p1 = {pos = {-100,-100}, v = {0,0}, a = {0,0} }
	local p2 = {pos = {100,-100}, v = {0,0}, a = {0,0} }
	local p3 = {pos = {-100,100}, v = {0,0}, a = {0,0} }
	local p4 = {pos = {100,100}, v = {0,0}, a = {0,0} }
	
	table.insert( points, p1 )
	table.insert( points, p2 )
	table.insert( points, p3 )
	table.insert( points, p4 )

	M_inv = matrix.invert( M )

	love.graphics.setPointSize( 6 )
	love.graphics.setPointStyle( "smooth" )
	love.graphics.setLineWidth( 2 )
	love.graphics.setLineStyle( "smooth" )
end

function love.draw()

	--love.graphics.translate( love.graphics.getWidth()/2, love.graphics.getHeight()/2 )
	for k = 1, #pX do
		love.graphics.setColor( 255,128,128,255 )
		for l = k+1, #pX do
			love.graphics.line( pX[k][1], pY[k][1], pX[l][1], pY[l][1] )
		end
		love.graphics.setColor( 255,255,255,255 )
		love.graphics.point( pX[k][1], pY[k][1] )
	end

	if love.mouse.isDown('l') then
		love.graphics.setColor( 128,255,128,255 )
		love.graphics.line( love.mouse.getX(), love.mouse.getY(), pX[1][1], pY[1][1] )
	elseif love.mouse.isDown('r') then
		love.graphics.setColor( 255,0,0,255 )
		love.graphics.line( love.mouse.getX(), love.mouse.getY(), pX[1][1], pY[1][1] )
	end

	drawUI()
end


function love.update( dt )

	if love.mouse.isDown('l') then
		-- Attract:
		fX[1][1] = love.mouse.getX() - pX[1][1]
		fY[1][1] = love.mouse.getY() - pY[1][1]
	elseif love.mouse.isDown('r') then
		-- Repell:
		fX[1][1] = - (love.mouse.getX() - pX[1][1])
		fY[1][1] = - (love.mouse.getY() - pY[1][1])
	else
		fX[1][1] = 0
		fY[1][1] = 0
		fX[2][1] = 0
		fY[2][1] = 0
		fX[3][1] = 0
		fY[3][1] = 0
		fX[4][1] = 0
		fY[4][1] = 0
	end

	local h = dt
	for k, p in ipairs( points ) do
		--[[print("vX:", vX)
		print("fX:", fX)
		print("pX:", pX)]]
		local vX_new = vX + h*M_inv*(fX - c*l0X - D*vX - S*pX)
		local pX_new = pX + h*vX

		vX = vX_new
		pX = pX_new

		local vY_new = vY + h*M_inv*(fY - c*l0Y - D*vY - S*pY)
		local pY_new = pY + h*vY

		vY = vY_new
		pY = pY_new
	end
end

function drawUI()
	local y, dy = 20, 15
	local x = 250
	-- Draw the controls:
	love.graphics.setColor( 255, 200, 0, 255 )
	love.graphics.printf( "Left click:", 20, y, x, "right" )
	y = y + dy
	love.graphics.printf( "Right click:", 20, y, x, "right" )
	y = y + dy
	love.graphics.printf( "Mass of points (q/a):", 20, y, x, "right" )
	y = y + dy
	love.graphics.printf( "Spring constant of connections: (w/s)", 20, y, x, "right" )
	y = y + dy
	love.graphics.printf( "Damping constant of connections (e/d):", 20, y, x, "right" )
	y = y + dy

	love.graphics.setColor( 255, 255, 255, 255 )
	y = 20
	love.graphics.printf( "Attracting force", x + 30, y, 170, "left" )
	y = y + dy
	love.graphics.printf( "Repelling force", x + 30, y, 170, "left" )
	y = y + dy
	love.graphics.printf( m, x + 30, y, 200, "left" )
	y = y + dy
	love.graphics.printf( c, x + 30, y, 200, "left" )
	y = y + dy
	love.graphics.printf( d, x + 30, y, 200, "left" )
	y = y + dy
end



function love.keypressed( key, repeated )
	if key == 'q' then
		m = m*2
	elseif key == 'a' then
		m = m*0.5
	elseif key == 'w' then
		c = c*2
	elseif key == 's' then
		c = c*0.5
	elseif key == 'e' then
		d = d*2
	elseif key == 'd' then
		d = d*0.5
	end

	-- Recreate matrices:
	M = matrix {
		{ m, 0, 0, 0 },
		{ 0, m, 0, 0 },
		{ 0, 0, m, 0 },
		{ 0, 0, 0, m } }
	D = matrix {
		{ 3*d, -d, -d, -d },
		{ -d, 3*d, -d, -d },
		{ -d, -d, 3*d, -d },
		{ -d, -d, -d, 3*d } }
	S = matrix {
		{ 3*c, -c, -c, -c },
		{ -c, 3*c, -c, -c },
		{ -c, -c, 3*c, -c },
		{ -c, -c, -c, 3*c } }
end
